"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.operate = exports.Subscriber = void 0;
var isFunction_1 = require("./util/isFunction");
var Subscription_1 = require("./Subscription");
var config_1 = require("./config");
var reportUnhandledError_1 = require("./util/reportUnhandledError");
var NotificationFactories_1 = require("./NotificationFactories");
var timeoutProvider_1 = require("./scheduler/timeoutProvider");
var Subscriber = (function (_super) {
    __extends(Subscriber, _super);
    function Subscriber(destination, overrides) {
        var _this = this;
        var _a, _b, _c, _d;
        _this = _super.call(this) || this;
        _this.isStopped = false;
        _this._nextOverride = null;
        _this._errorOverride = null;
        _this._completeOverride = null;
        _this._onFinalize = null;
        _this.destination = destination instanceof Subscriber ? destination : createSafeObserver(destination);
        _this._nextOverride = (_a = overrides === null || overrides === void 0 ? void 0 : overrides.next) !== null && _a !== void 0 ? _a : null;
        _this._errorOverride = (_b = overrides === null || overrides === void 0 ? void 0 : overrides.error) !== null && _b !== void 0 ? _b : null;
        _this._completeOverride = (_c = overrides === null || overrides === void 0 ? void 0 : overrides.complete) !== null && _c !== void 0 ? _c : null;
        _this._onFinalize = (_d = overrides === null || overrides === void 0 ? void 0 : overrides.finalize) !== null && _d !== void 0 ? _d : null;
        _this._next = _this._nextOverride ? overrideNext : _this._next;
        _this._error = _this._errorOverride ? overrideError : _this._error;
        _this._complete = _this._completeOverride ? overrideComplete : _this._complete;
        if (hasAddAndUnsubscribe(destination)) {
            destination.add(_this);
        }
        return _this;
    }
    Subscriber.prototype.next = function (value) {
        if (this.isStopped) {
            handleStoppedNotification((0, NotificationFactories_1.nextNotification)(value), this);
        }
        else {
            this._next(value);
        }
    };
    Subscriber.prototype.error = function (err) {
        if (this.isStopped) {
            handleStoppedNotification((0, NotificationFactories_1.errorNotification)(err), this);
        }
        else {
            this.isStopped = true;
            this._error(err);
        }
    };
    Subscriber.prototype.complete = function () {
        if (this.isStopped) {
            handleStoppedNotification(NotificationFactories_1.COMPLETE_NOTIFICATION, this);
        }
        else {
            this.isStopped = true;
            this._complete();
        }
    };
    Subscriber.prototype.unsubscribe = function () {
        var _a;
        if (!this.closed) {
            this.isStopped = true;
            _super.prototype.unsubscribe.call(this);
            (_a = this._onFinalize) === null || _a === void 0 ? void 0 : _a.call(this);
        }
    };
    Subscriber.prototype._next = function (value) {
        this.destination.next(value);
    };
    Subscriber.prototype._error = function (err) {
        try {
            this.destination.error(err);
        }
        finally {
            this.unsubscribe();
        }
    };
    Subscriber.prototype._complete = function () {
        try {
            this.destination.complete();
        }
        finally {
            this.unsubscribe();
        }
    };
    return Subscriber;
}(Subscription_1.Subscription));
exports.Subscriber = Subscriber;
function overrideNext(value) {
    try {
        this._nextOverride(value);
    }
    catch (error) {
        this.destination.error(error);
    }
}
function overrideError(err) {
    try {
        this._errorOverride(err);
    }
    catch (error) {
        this.destination.error(error);
    }
    finally {
        this.unsubscribe();
    }
}
function overrideComplete() {
    try {
        this._completeOverride();
    }
    catch (error) {
        this.destination.error(error);
    }
    finally {
        this.unsubscribe();
    }
}
var ConsumerObserver = (function () {
    function ConsumerObserver(partialObserver) {
        this.partialObserver = partialObserver;
    }
    ConsumerObserver.prototype.next = function (value) {
        var partialObserver = this.partialObserver;
        if (partialObserver.next) {
            try {
                partialObserver.next(value);
            }
            catch (error) {
                (0, reportUnhandledError_1.reportUnhandledError)(error);
            }
        }
    };
    ConsumerObserver.prototype.error = function (err) {
        var partialObserver = this.partialObserver;
        if (partialObserver.error) {
            try {
                partialObserver.error(err);
            }
            catch (error) {
                (0, reportUnhandledError_1.reportUnhandledError)(error);
            }
        }
        else {
            (0, reportUnhandledError_1.reportUnhandledError)(err);
        }
    };
    ConsumerObserver.prototype.complete = function () {
        var partialObserver = this.partialObserver;
        if (partialObserver.complete) {
            try {
                partialObserver.complete();
            }
            catch (error) {
                (0, reportUnhandledError_1.reportUnhandledError)(error);
            }
        }
    };
    return ConsumerObserver;
}());
function createSafeObserver(observerOrNext) {
    return new ConsumerObserver(!observerOrNext || (0, isFunction_1.isFunction)(observerOrNext) ? { next: observerOrNext !== null && observerOrNext !== void 0 ? observerOrNext : undefined } : observerOrNext);
}
function handleStoppedNotification(notification, subscriber) {
    var onStoppedNotification = config_1.config.onStoppedNotification;
    onStoppedNotification && timeoutProvider_1.timeoutProvider.setTimeout(function () { return onStoppedNotification(notification, subscriber); });
}
function hasAddAndUnsubscribe(value) {
    return value && (0, isFunction_1.isFunction)(value.unsubscribe) && (0, isFunction_1.isFunction)(value.add);
}
function operate(_a) {
    var destination = _a.destination, subscriberOverrides = __rest(_a, ["destination"]);
    return new Subscriber(destination, subscriberOverrides);
}
exports.operate = operate;
//# sourceMappingURL=Subscriber.js.map