"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.audit = void 0;
var Subscriber_1 = require("../Subscriber");
var Observable_1 = require("../Observable");
var from_1 = require("../observable/from");
function audit(durationSelector) {
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var hasValue = false;
            var lastValue = null;
            var durationSubscriber = null;
            var isComplete = false;
            var endDuration = function () {
                durationSubscriber === null || durationSubscriber === void 0 ? void 0 : durationSubscriber.unsubscribe();
                durationSubscriber = null;
                if (hasValue) {
                    hasValue = false;
                    var value = lastValue;
                    lastValue = null;
                    destination.next(value);
                }
                isComplete && destination.complete();
            };
            var cleanupDuration = function () {
                durationSubscriber = null;
                isComplete && destination.complete();
            };
            source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) {
                    hasValue = true;
                    lastValue = value;
                    if (!durationSubscriber) {
                        (0, from_1.from)(durationSelector(value)).subscribe((durationSubscriber = (0, Subscriber_1.operate)({
                            destination: destination,
                            next: endDuration,
                            complete: cleanupDuration,
                        })));
                    }
                },
                complete: function () {
                    isComplete = true;
                    (!hasValue || !durationSubscriber || durationSubscriber.closed) && destination.complete();
                },
            }));
        });
    };
}
exports.audit = audit;
//# sourceMappingURL=audit.js.map