"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debounce = void 0;
var Subscriber_1 = require("../Subscriber");
var Observable_1 = require("../Observable");
var noop_1 = require("../util/noop");
var from_1 = require("../observable/from");
function debounce(durationSelector) {
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var hasValue = false;
            var lastValue = null;
            var durationSubscriber = null;
            var emit = function () {
                durationSubscriber === null || durationSubscriber === void 0 ? void 0 : durationSubscriber.unsubscribe();
                durationSubscriber = null;
                if (hasValue) {
                    hasValue = false;
                    var value = lastValue;
                    lastValue = null;
                    destination.next(value);
                }
            };
            source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) {
                    durationSubscriber === null || durationSubscriber === void 0 ? void 0 : durationSubscriber.unsubscribe();
                    hasValue = true;
                    lastValue = value;
                    durationSubscriber = (0, Subscriber_1.operate)({ destination: destination, next: emit, complete: noop_1.noop });
                    (0, from_1.from)(durationSelector(value)).subscribe(durationSubscriber);
                },
                complete: function () {
                    emit();
                    destination.complete();
                },
                finalize: function () {
                    lastValue = durationSubscriber = null;
                },
            }));
        });
    };
}
exports.debounce = debounce;
//# sourceMappingURL=debounce.js.map