"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debounceTime = void 0;
var async_1 = require("../scheduler/async");
var Observable_1 = require("../Observable");
var Subscriber_1 = require("../Subscriber");
function debounceTime(dueTime, scheduler) {
    if (scheduler === void 0) { scheduler = async_1.asyncScheduler; }
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var activeTask = null;
            var lastValue = null;
            var lastTime = null;
            var scheduling = false;
            var emit = function () {
                if (scheduling || activeTask) {
                    if (activeTask) {
                        activeTask.unsubscribe();
                        activeTask = null;
                    }
                    var value = lastValue;
                    lastValue = null;
                    destination.next(value);
                }
            };
            function emitWhenIdle() {
                var targetTime = lastTime + dueTime;
                var now = scheduler.now();
                if (now < targetTime) {
                    activeTask = this.schedule(undefined, targetTime - now);
                    destination.add(activeTask);
                    return;
                }
                emit();
            }
            source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) {
                    lastValue = value;
                    lastTime = scheduler.now();
                    if (!activeTask) {
                        scheduling = true;
                        activeTask = scheduler.schedule(emitWhenIdle, dueTime);
                        scheduling = false;
                        destination.add(activeTask);
                    }
                },
                complete: function () {
                    emit();
                    destination.complete();
                },
                finalize: function () {
                    lastValue = activeTask = null;
                },
            }));
        });
    };
}
exports.debounceTime = debounceTime;
//# sourceMappingURL=debounceTime.js.map