"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.distinct = void 0;
var Observable_1 = require("../Observable");
var Subscriber_1 = require("../Subscriber");
var noop_1 = require("../util/noop");
var from_1 = require("../observable/from");
function distinct(keySelector, flushes) {
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var distinctKeys = new Set();
            source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) {
                    var key = keySelector ? keySelector(value) : value;
                    if (!distinctKeys.has(key)) {
                        distinctKeys.add(key);
                        destination.next(value);
                    }
                },
            }));
            flushes && (0, from_1.from)(flushes).subscribe((0, Subscriber_1.operate)({ destination: destination, next: function () { return distinctKeys.clear(); }, complete: noop_1.noop }));
        });
    };
}
exports.distinct = distinct;
//# sourceMappingURL=distinct.js.map