"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.distinctUntilChanged = void 0;
var identity_1 = require("../util/identity");
var Observable_1 = require("../Observable");
var Subscriber_1 = require("../Subscriber");
function distinctUntilChanged(comparator, keySelector) {
    if (keySelector === void 0) { keySelector = identity_1.identity; }
    comparator = comparator !== null && comparator !== void 0 ? comparator : defaultCompare;
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var previousKey;
            var first = true;
            source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) {
                    var currentKey = keySelector(value);
                    if (first || !comparator(previousKey, currentKey)) {
                        first = false;
                        previousKey = currentKey;
                        destination.next(value);
                    }
                },
            }));
        });
    };
}
exports.distinctUntilChanged = distinctUntilChanged;
function defaultCompare(a, b) {
    return a === b;
}
//# sourceMappingURL=distinctUntilChanged.js.map