"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.retry = void 0;
var Observable_1 = require("../Observable");
var Subscriber_1 = require("../Subscriber");
var identity_1 = require("../util/identity");
var timer_1 = require("../observable/timer");
var from_1 = require("../observable/from");
function retry(configOrCount) {
    if (configOrCount === void 0) { configOrCount = Infinity; }
    var config;
    if (configOrCount && typeof configOrCount === 'object') {
        config = configOrCount;
    }
    else {
        config = {
            count: configOrCount,
        };
    }
    var _a = config.count, count = _a === void 0 ? Infinity : _a, delay = config.delay, _b = config.resetOnSuccess, resetOnSuccess = _b === void 0 ? false : _b;
    return count <= 0
        ? identity_1.identity
        : function (source) {
            return new Observable_1.Observable(function (destination) {
                var soFar = 0;
                var innerSub;
                var subscribeForRetry = function () {
                    var syncUnsub = false;
                    innerSub = source.subscribe((0, Subscriber_1.operate)({
                        destination: destination,
                        next: function (value) {
                            if (resetOnSuccess) {
                                soFar = 0;
                            }
                            destination.next(value);
                        },
                        error: function (err) {
                            if (soFar++ < count) {
                                var resub_1 = function () {
                                    if (innerSub) {
                                        innerSub.unsubscribe();
                                        innerSub = null;
                                        subscribeForRetry();
                                    }
                                    else {
                                        syncUnsub = true;
                                    }
                                };
                                if (delay != null) {
                                    var notifier = typeof delay === 'number' ? (0, timer_1.timer)(delay) : (0, from_1.from)(delay(err, soFar));
                                    var notifierSubscriber_1 = (0, Subscriber_1.operate)({
                                        destination: destination,
                                        next: function () {
                                            notifierSubscriber_1.unsubscribe();
                                            resub_1();
                                        },
                                        complete: function () {
                                            destination.complete();
                                        },
                                    });
                                    notifier.subscribe(notifierSubscriber_1);
                                }
                                else {
                                    resub_1();
                                }
                            }
                            else {
                                destination.error(err);
                            }
                        },
                    }));
                    if (syncUnsub) {
                        innerSub.unsubscribe();
                        innerSub = null;
                        subscribeForRetry();
                    }
                };
                subscribeForRetry();
            });
        };
}
exports.retry = retry;
//# sourceMappingURL=retry.js.map