"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.skipLast = void 0;
var identity_1 = require("../util/identity");
var Observable_1 = require("../Observable");
var Subscriber_1 = require("../Subscriber");
function skipLast(skipCount) {
    return skipCount <= 0
        ?
            identity_1.identity
        : function (source) {
            return new Observable_1.Observable(function (destination) {
                var ring = new Array(skipCount);
                var seen = 0;
                source.subscribe((0, Subscriber_1.operate)({
                    destination: destination,
                    next: function (value) {
                        var valueIndex = seen++;
                        if (valueIndex < skipCount) {
                            ring[valueIndex] = value;
                        }
                        else {
                            var index = valueIndex % skipCount;
                            var oldValue = ring[index];
                            ring[index] = value;
                            destination.next(oldValue);
                        }
                    },
                }));
                return function () {
                    ring = null;
                };
            });
        };
}
exports.skipLast = skipLast;
//# sourceMappingURL=skipLast.js.map