"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tap = void 0;
var isFunction_1 = require("../util/isFunction");
var Observable_1 = require("../Observable");
var Subscriber_1 = require("../Subscriber");
var identity_1 = require("../util/identity");
function tap(observerOrNext) {
    var tapObserver = (0, isFunction_1.isFunction)(observerOrNext) ? { next: observerOrNext } : observerOrNext;
    return tapObserver
        ? function (source) {
            return new Observable_1.Observable(function (destination) {
                var _a;
                (_a = tapObserver.subscribe) === null || _a === void 0 ? void 0 : _a.call(tapObserver);
                var isUnsub = true;
                source.subscribe((0, Subscriber_1.operate)({
                    destination: destination,
                    next: function (value) {
                        var _a;
                        (_a = tapObserver.next) === null || _a === void 0 ? void 0 : _a.call(tapObserver, value);
                        destination.next(value);
                    },
                    error: function (err) {
                        var _a;
                        isUnsub = false;
                        (_a = tapObserver.error) === null || _a === void 0 ? void 0 : _a.call(tapObserver, err);
                        destination.error(err);
                    },
                    complete: function () {
                        var _a;
                        isUnsub = false;
                        (_a = tapObserver.complete) === null || _a === void 0 ? void 0 : _a.call(tapObserver);
                        destination.complete();
                    },
                    finalize: function () {
                        var _a, _b;
                        if (isUnsub) {
                            (_a = tapObserver.unsubscribe) === null || _a === void 0 ? void 0 : _a.call(tapObserver);
                        }
                        (_b = tapObserver.finalize) === null || _b === void 0 ? void 0 : _b.call(tapObserver);
                    },
                }));
            });
        }
        :
            identity_1.identity;
}
exports.tap = tap;
//# sourceMappingURL=tap.js.map