"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.throttle = void 0;
var Observable_1 = require("../Observable");
var Subscriber_1 = require("../Subscriber");
var from_1 = require("../observable/from");
function throttle(durationSelector, config) {
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var _a = config !== null && config !== void 0 ? config : {}, _b = _a.leading, leading = _b === void 0 ? true : _b, _c = _a.trailing, trailing = _c === void 0 ? false : _c;
            var hasValue = false;
            var sendValue = null;
            var throttled = null;
            var isComplete = false;
            var endThrottling = function () {
                throttled === null || throttled === void 0 ? void 0 : throttled.unsubscribe();
                throttled = null;
                if (trailing) {
                    send();
                    isComplete && destination.complete();
                }
            };
            var cleanupThrottling = function () {
                throttled = null;
                isComplete && destination.complete();
            };
            var startThrottle = function (value) {
                return (throttled = (0, from_1.from)(durationSelector(value)).subscribe((0, Subscriber_1.operate)({ destination: destination, next: endThrottling, complete: cleanupThrottling })));
            };
            var send = function () {
                if (hasValue) {
                    hasValue = false;
                    var value = sendValue;
                    sendValue = null;
                    destination.next(value);
                    !isComplete && startThrottle(value);
                }
            };
            source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) {
                    hasValue = true;
                    sendValue = value;
                    !(throttled && !throttled.closed) && (leading ? send() : startThrottle(value));
                },
                complete: function () {
                    isComplete = true;
                    !(trailing && hasValue && throttled && !throttled.closed) && destination.complete();
                },
            }));
        });
    };
}
exports.throttle = throttle;
//# sourceMappingURL=throttle.js.map