"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.timeout = exports.TimeoutError = void 0;
var async_1 = require("../scheduler/async");
var isDate_1 = require("../util/isDate");
var Observable_1 = require("../Observable");
var from_1 = require("../observable/from");
var Subscriber_1 = require("../Subscriber");
var executeSchedule_1 = require("../util/executeSchedule");
var TimeoutError = (function (_super) {
    __extends(TimeoutError, _super);
    function TimeoutError(info) {
        if (info === void 0) { info = null; }
        var _this = _super.call(this, 'Timeout has occurred') || this;
        _this.info = info;
        _this.name = 'TimeoutError';
        return _this;
    }
    return TimeoutError;
}(Error));
exports.TimeoutError = TimeoutError;
function timeout(config, schedulerArg) {
    var _a = ((0, isDate_1.isValidDate)(config) ? { first: config } : typeof config === 'number' ? { each: config } : config), first = _a.first, each = _a.each, _b = _a.with, _with = _b === void 0 ? timeoutErrorFactory : _b, _c = _a.scheduler, scheduler = _c === void 0 ? schedulerArg !== null && schedulerArg !== void 0 ? schedulerArg : async_1.asyncScheduler : _c, _d = _a.meta, meta = _d === void 0 ? null : _d;
    if (first == null && each == null) {
        throw new TypeError('No timeout provided.');
    }
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var originalSourceSubscription;
            var timerSubscription;
            var lastValue = null;
            var seen = 0;
            var startTimer = function (delay) {
                timerSubscription = (0, executeSchedule_1.executeSchedule)(destination, scheduler, function () {
                    try {
                        originalSourceSubscription.unsubscribe();
                        (0, from_1.from)(_with({
                            meta: meta,
                            lastValue: lastValue,
                            seen: seen,
                        })).subscribe(destination);
                    }
                    catch (err) {
                        destination.error(err);
                    }
                }, delay);
            };
            originalSourceSubscription = source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) {
                    timerSubscription === null || timerSubscription === void 0 ? void 0 : timerSubscription.unsubscribe();
                    seen++;
                    destination.next((lastValue = value));
                    each > 0 && startTimer(each);
                },
                finalize: function () {
                    if (!(timerSubscription === null || timerSubscription === void 0 ? void 0 : timerSubscription.closed)) {
                        timerSubscription === null || timerSubscription === void 0 ? void 0 : timerSubscription.unsubscribe();
                    }
                    lastValue = null;
                },
            }));
            !seen && startTimer(first != null ? (typeof first === 'number' ? first : +first - scheduler.now()) : each);
        });
    };
}
exports.timeout = timeout;
function timeoutErrorFactory(info) {
    throw new TimeoutError(info);
}
//# sourceMappingURL=timeout.js.map