"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.window = void 0;
var Observable_1 = require("../Observable");
var Subject_1 = require("../Subject");
var Subscriber_1 = require("../Subscriber");
var noop_1 = require("../util/noop");
var from_1 = require("../observable/from");
function window(windowBoundaries) {
    return function (source) {
        return new Observable_1.Observable(function (destination) {
            var windowSubject = new Subject_1.Subject();
            destination.next(windowSubject.asObservable());
            var errorHandler = function (err) {
                windowSubject.error(err);
                destination.error(err);
            };
            source.subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function (value) { return windowSubject === null || windowSubject === void 0 ? void 0 : windowSubject.next(value); },
                complete: function () {
                    windowSubject.complete();
                    destination.complete();
                },
                error: errorHandler,
            }));
            (0, from_1.from)(windowBoundaries).subscribe((0, Subscriber_1.operate)({
                destination: destination,
                next: function () {
                    windowSubject.complete();
                    destination.next((windowSubject = new Subject_1.Subject()));
                },
                complete: noop_1.noop,
                error: errorHandler,
            }));
            return function () {
                windowSubject === null || windowSubject === void 0 ? void 0 : windowSubject.unsubscribe();
                windowSubject = null;
            };
        });
    };
}
exports.window = window;
//# sourceMappingURL=window.js.map